@@ -1,32 +1,12 @@
 #####RED - IUCN Redlisting Tools
-#####Version 1.2.1 (2017-10-05)
+#####Version 1.3.0 (2017-10-17)
 #####By Pedro Cardoso
 #####Maintainer: pedro.cardoso@helsinki.fi
-#####Reference: Cardoso, P.(in prep.) An R package to facilitate species red list assessments according to the IUCN criteria.
-#####Changed from v1.2.0:
-#####More efficient calculation of AOO for large areas
-#####More eficient thinning for species with many records
-#####Centroid at outlier graph
-#####on.exit at red.setup() to avoid unexpected behavior if function fails
-
-#Todo:
-# rli*: add trees (for phylogenetic and functional diversity)
-# map.habitat: automatically select habitat or range
-# map.change: new function and automated download of future climate layers and forest change across years
-# map.sdm: multicore p multiple runs
-# kbas: implement
-
-#####data origins:
-#####climate -> Fick, S.E. & Hijmans, R.J. (2017) Worldclim 2: new 1-km spatial resolution climate surfaces for global land areas. International Journal of Climatology, in press.
-#####altitude -> Farr, T. G., et al. (2007), The Shuttle Radar Topography Mission, Rev. Geophys., 45, RG2004
-#####landcover -> Tuanmu, M.-N. & Jetz, W. (2014) A global 1-km consensus land-cover product for biodiversity and ecosystem modeling. Global Ecology and Biogeography, 23: 1031-1045.
-
-#####RED Stats:
-#####library("cranlogs")
-#####day <- cran_downloads(package = "red", from = "2016-08-19", to = "2017-06-21")
-#####group <- matrix(day$count, 100, byrow = TRUE)
-#####plot(rowSums(group), type = "n")
-#####lines(rowSums(group))
+#####Reference: Cardoso, P.(subm.) An R package to facilitate species red list assessments according to the IUCN criteria.
+#####Changed from v1.2.1:
+#####rli.map: added function
+#####eoo, aoo: can now use probabilistic maps and output confidence limits.
+#####rli, rli.multi, rli.sample: added trees for phylogenetic and functional diversity and now accept NA values.
 
 #####required packages
 library("BAT")
@@ -105,7 +85,9 @@ find.categorical <- function(layers){
 
 ##basic function to calculate the rli of any group of species
 rli.calc <- function(spData, tree = NULL, boot = FALSE, runs = 1000){
-  if(max(spData) > 5){                                ##if letters are given, convert to [0,1]
+  if(all(is.na(spData)))
+    return(NA)
+  if(max(spData, na.rm = TRUE) > 5){                                ##if letters are given, convert to [0,1]
     spData <- replace(spData, which(spData == "EX" ), 0)
     spData <- replace(spData, which(spData == "EW" ), 0)
     spData <- replace(spData, which(spData == "RE" ), 0)
@@ -114,20 +96,19 @@ rli.calc <- function(spData, tree = NULL, boot = FALSE, runs = 1000){
     spData <- replace(spData, which(spData == "VU" ), 0.6)
     spData <- replace(spData, which(spData == "NT" ), 0.8)
     spData <- replace(spData, which(spData == "LC" ), 1)
-    spData <- replace(spData, which(spData == "DD" ), 2)
+    spData <- replace(spData, which(spData == "DD" ), NA)
     spData <- as.numeric(spData)
-    spData <- subset(spData, spData < 2)
-  } else if (max(spData) > 1){                       ##if a scale [0,5] is given, convert to [0,1]
+  } else if (max(spData, na.rm = TRUE) > 1){                       ##if a scale [0,5] is given, convert to [0,1]
     spData <- 1 - spData / 5
   }
   if(is.null(tree)){                           ##if not weighted by PD or FD
     if(!boot){                                 ##if no bootstrap to be made
-      return (mean(spData))
+      return (mean(spData, na.rm = TRUE))
     } else {
       run <- rep(NA, runs)
       for(i in 1:runs){
-        rnd <- sample(spData, length(spData), replace = TRUE) ##bootstrap
-        run[i] <- mean(rnd)
+        rnd <- sample(spData, replace = TRUE) ##bootstrap
+        run[i] <- mean(rnd, na.rm = TRUE)
       }
       res <- matrix(quantile(run, c(0.025, 0.5, 0.975)), nrow = 1)
       colnames(res) <- c("LowCL", "Median", "UpCL")
@@ -136,15 +117,20 @@ rli.calc <- function(spData, tree = NULL, boot = FALSE, runs = 1000){
   } else {                                     ##if weighted by PD or FD
     comm <- matrix(1, nrow = 2, ncol = length(spData))
     contrib <- BAT::contribution(comm, tree, relative = TRUE)[1,]
+    contrib <- contrib/sum(contrib[!is.na(spData)]) #needed to standardize the contribution by the total contribution of species living in the community
     if(!boot){                                 ##if no bootstrap to be made
-      return (sum(contrib * spData))
+      return(sum(spData * contrib, na.rm = TRUE))
     } else {
+      run <- rep(NA, runs)
       for(i in 1:runs){
-        rnd <- sample(spData, length(spData), replace = TRUE) ##bootstrap
-        for(i in 1:length(rnd))
-          run[i] <- run[i] + contrib[rnd[i]] * spData[rnd[i]]
+        rndSpp <- sample(length(spData), replace = TRUE)
+        rndComm <- spData[rndSpp]
+        rndContrib <- contrib[rndSpp]/sum(contrib[rndSpp])
+        run[i] <- sum(rndComm * rndContrib, na.rm = TRUE)
       }
-      return(quantile(run, c(0.025, 0.5, 0.975)))
+        res <- matrix(quantile(run, c(0.025, 0.5, 0.975)), nrow = 1)
+        colnames(res) <- c("LowCL", "Median", "UpCL")
+        return(res)
     }
   }
 }
@@ -704,7 +690,7 @@ map.sdm <- function(longlat, layers, error = NULL, categorical = NULL, thres = 0
     upMap <- reclassify(runMap, matrix(c(0,0.025,0,0.025,1,1), ncol = 3, byrow = TRUE))
     consensusMap <- reclassify(runMap, matrix(c(0,0.499,0,0.499,1,1), ncol = 3, byrow = TRUE))
     downMap <- reclassify(runMap, matrix(c(0,0.975,0,0.975,1,1), ncol = 3, byrow = TRUE))
-    if(mcp && aoo(consensusMap) >= 4)
+    if(mcp && aoo(runMap[2]) >= 4)
       consensusMap <- map.habitat(longlat, consensusMap, mcp = TRUE, eval = FALSE)
 
     if(eval){
@@ -868,104 +854,6 @@ map.points <- function(longlat, layers, eval = TRUE){
   }
 }
 
-## #' Predict species distribution change in time.
-## #' @description Prediction and projection in time of potential species distribution using maximum entropy (maxent).
-## #' @param longlat Matrix of longitude and latitude or eastness and northness (two columns in this order) of each occurrence record.
-## #' @param layers Raster* object as defined by package raster, used for modelling current conditions.
-## #' @param projectLayers Raster* object as defined by package raster, used for projection into the future.
-## #' @param bg Background data as a matrix of longitude and latitude (two columns). If not defined 1000 points will be randomly selected.
-## #' @param categorical Vector of layer indices of categorical (as opposed to quantitative) data. If NULL the package will try to find them automatically based on the data itself.
-## #' @param thres Threshold of logistic output used for conversion of probabilistic to binary (presence/absence) maps. If 1 this will be the value that maximizes the sum of sensitivity and specificity.
-## #' @param polygon Used for a precautionary approach. If TRUE, all areas predicted as present but outside the minimum convex hull polygon encompassing all occurrence records are converted to absence. Only cells connected to other areas inside the polygon are kept for both present and future projections.
-## #' @param runs If <= 0 no ensemble modelling is performed. If > 0, ensemble modelling with n runs is made. For each run a bootstrap (random sampling with replacement) of all occurrence records and a new set of 1000 background points are chosen.
-## #' @details Builds maxent (maximum entropy) species distribution models (Phillips et al. 2004, 2006; Elith et al. 2011) using function maxent from R package dismo (Hijmans et al. 2017) for both the present and future.
-## #' @return Either three or six RasterLayer (depending if ensemble modelling is performed, in which case the second trio are probabilistic maps from all the runs) and a matrix with Present AOO, Future AOO, Gain, Keep and Loss.
-## #' @references Hijmans, R.J., Phillips, S., Leathwick, J., Elith, J. (2017) dismo: Species Distribution Modeling. R package version 1.1-4. https://CRAN.R-project.org/package=dismo
-## #' @references Phillips, S.J., Dudik, M., Schapire, R.E. (2004) A maximum entropy approach to species distribution modeling. Proceedings of the Twenty-First International Conference on Machine Learning. p. 655-662.
-## #' @references Phillips, S.J., Anderson, R.P., Schapire, R.E. (2006) Maximum entropy modeling of species geographic distributions. Ecological Modelling 190:231-259.
-## #' @references Elith, J., Phillips, S.J., Hastie, T., Dudik, M., Chee, Y.E., Yates, C.J. (2011) A statistical explanation of MaxEnt for ecologists. Diversity and Distributions 17:43-57.
-## #' @export
-## map.change <- function(longlat, layers, projectLayers, bg = NULL, categorical = NULL, thres = 1, polygon = FALSE, runs = 0){
-##   options(warn=-1)
-#   ##if ensemble modelling is to be done
-#   if(runs > 0){
-#     runEval = matrix(NA, nrow = 1, ncol = 5)
-#     runMap <- rasterize(longlat, layers[[1]], field = 0, background = 0)
-#     runMap <- raster::stack(runMap, runMap, runMap)
-#     runMap01 <- runMap
-#     pb <- txtProgressBar(min = 0, max = runs, style = 3)
-#     for(i in 1:runs){
-#       runData <- longlat[sample(nrow(longlat), replace = TRUE),]
-#       thisRun <- map.change(runData, layers, projectLayers, bg, categorical, thres, polygon, runs = 0)
-#       runEval <- rbind(runEval, thisRun[[4]])
-#       thisRun <- list(thisRun[[1]], thisRun[[2]], thisRun[[3]])
-#       for(j in 1:3)
-#         runMap[[j]] <- runMap[[j]] + thisRun[[j]] / runs
-#       setTxtProgressBar(pb, i)
-#     }
-#     for(i in 1:2)
-#       runMap01[[i]] <- reclassify(runMap[[i]], matrix(c(0,0.5,0,0.5,1,1), ncol = 3, byrow = TRUE))
-#     runMap01[[3]] <- runMap01[[2]] * 2 - runMap01[[1]] ##gain = 2, kept = 1, loss = -1, never exists = 0
-#     runEval <- runEval[-1,]
-#     clEval <- matrix(NA, nrow = 4, ncol = 5)
-#     colnames(clEval) <- c("Present AOO", "Future AOO", "Gain", "Keep", "Loss")
-#     rownames(clEval) <- c("Aggregate", "LowCL", "Median", "UpCL")
-#     clEval[1,1] <- aoo(runMap01[[1]])
-#     clEval[1,2] <- aoo(runMap01[[2]])
-#     clEval[1,3] <- cellStats((raster::area(runMap01[[3]]) * subs(runMap01[[3]], as.data.frame(matrix(c(0,0,1,0,-1,0,2,1), ncol = 2, byrow = TRUE)))),sum)
-#     clEval[1,4] <- cellStats((raster::area(runMap01[[3]]) * subs(runMap01[[3]], as.data.frame(matrix(c(0,0,1,1,-1,0,2,0), ncol = 2, byrow = TRUE)))),sum)
-#     clEval[1,5] <- cellStats((raster::area(runMap01[[3]]) * subs(runMap01[[3]], as.data.frame(matrix(c(0,0,1,0,-1,1,2,0), ncol = 2, byrow = TRUE)))),sum)
-#     clEval[2,] <- apply(runEval, 2,  quantile, probs= 0.025, na.rm = TRUE)
-#     clEval[3,] <- apply(runEval, 2,  quantile, probs= 0.5, na.rm = TRUE)
-#     clEval[4,] <- apply(runEval, 2,  quantile, probs= 0.975, na.rm = TRUE)
-#     options(warn=0)
-#     return(list(runMap01, runMap, clEval))
-#   }
-#
-#   ##if no background points are given randomly sample them
-#   if(is.null(bg))
-#     bg <- randomPoints(layers, 1000)                                ##extract background points (to use as absence)
-#   ##if no categorical variables are given try to figure out which
-#   if(is.null(categorical))
-#     categorical = find.categorical(layers)
-#
-#   model <- dismo::maxent(layers, longlat, a = bg, factors = categorical) ##build model
-#   present <- raster::predict(model, layers)                             ##do prediction for the present
-#   future <- raster::predict(model, projectLayers)
-#   e <- dismo::evaluate(longlat, bg, model, layers)                       ##do evaluation of model
-#   if(thres >= 1)
-#     thres <- threshold(e)$spec_sens                                   ##extract threshold from evaluation
-#   present <- reclassify(present, matrix(c(0,thres,0,thres,1,1), nrow=2, byrow = TRUE))  ##convert to presence/absence
-#   future <- reclassify(future, matrix(c(0,thres,0,thres,1,1), nrow=2, byrow = TRUE))  ##convert to presence/absence
-#
-#   if(polygon){                          ##if species is limited in dispersal ability
-#     vertices <- chull(longlat)
-#     vertices <- c(vertices, vertices[1])
-#     vertices <- longlat[vertices,]
-#     poly = Polygon(vertices)
-#     poly = Polygons(list(poly),1)
-#     poly = SpatialPolygons(list(poly))    ##original EOO, before modelling
-#     ##present
-#     patches <- clump(present, gaps=FALSE)       ##individual patches, numbered, present
-#     selPatches <- unique(extract(patches, poly, df = TRUE, weights = TRUE)$clumps) ##which patches are inside original EOO
-#     present <- replace(present, !(patches %in% selPatches), 0)
-#     ##future
-#     patches <- clump(future, gaps=FALSE)       ##individual patches, numbered, future
-#     selPatches <- unique(extract(patches, poly, df = TRUE, weights = TRUE)$clumps) ##which patches are inside original EOO
-#     future <- replace(future, !(patches %in% selPatches), 0)
-#   }
-#
-#   spDiff <- future * 2 - present ##gain = 2, kept = 1, loss = -1, never exists = 0
-#   txtChange <- rep(NA,5)
-#   names(txtChange) <- c("Present AOO", "Future AOO", "Gain", "Keep", "Loss")
-#   txtChange[1] <- cellStats((raster::area(present) * present), sum)
-#   txtChange[2] <- cellStats((raster::area(future) * future), sum)
-#   txtChange[3] <- cellStats((raster::area(spDiff) * subs(spDiff, as.data.frame(matrix(c(0,0,1,0,-1,0,2,1), ncol = 2, byrow = TRUE)))),sum)
-#   txtChange[4] <- cellStats((raster::area(spDiff) * subs(spDiff, as.data.frame(matrix(c(0,0,1,1,-1,0,2,0), ncol = 2, byrow = TRUE)))),sum)
-#   txtChange[5] <- cellStats((raster::area(spDiff) * subs(spDiff, as.data.frame(matrix(c(0,0,1,0,-1,1,2,0), ncol = 2, byrow = TRUE)))),sum)
-#   return(list(present, future, spDiff, txtChange))
-# }
-
 #' Species distributions made easy (multiple species).
 #' @description Single step for prediction of multiple species distributions. Output of maps (in pdf format), klms (for Google Earth) and relevant data (in csv format).
 #' @param longlat data.frame of taxon names, longitude and latitude or eastness and northness (three columns in this order) of each occurrence record.
@@ -1157,7 +1045,7 @@ map.draw <- function(longlat = NULL, layer, spName,  borders = FALSE, scale = TR
     scalebar(d = d, type="bar", divs = 2)
   }
   if (sites && !is.null(longlat))
-    points(longlat)
+    points(longlat, pch = 19)
   if (mcp){
     e <- rasterToPoints(layer, fun = function(dat){dat == 1})   ##convert raster to points
     vertices <- chull(e[,1], e[,2])
@@ -1174,9 +1062,9 @@ map.draw <- function(longlat = NULL, layer, spName,  borders = FALSE, scale = TR
 
 #' Extent of Occurrence (EOO).
 #' @description Calculates the Extent of Occurrence of a species based on either records or predicted distribution.
-#' @param spData One of three options: 1) matrix of longitude and latitude (two columns) of each occurrence record; 2) matrix of easting and northing (two columns, e.g. UTM) of each occurrence record in meters;  3) RasterLayer object of predicted distribution (0/1 values).
+#' @param spData spData One of three options: 1) matrix of longitude and latitude (two columns) of each occurrence record; 2) matrix of easting and northing (two columns, e.g. UTM) of each occurrence record in meters;  3) RasterLayer object of predicted distribution (either 0/1 or probabilistic values).
 #' @details EOO is calculated as the minimum convex polygon covering all known or predicted sites for the species.
-#' @return A single value in km2.
+#' @return A single value in km2 or a vector with lower confidence limit, consensus and upper confidence limit (probabilities 0.975, 0.5 and 0.025 respectively).
 #' @examples data(red.records)
 #' data(red.range)
 #' eoo(red.records)
@@ -1184,24 +1072,31 @@ map.draw <- function(longlat = NULL, layer, spName,  borders = FALSE, scale = TR
 #' @export
 eoo <- function(spData){
   if(class(spData) == "RasterLayer"){
-    if (raster::xmax(spData) <= 180) {  #if longlat data
-      e <- rasterToPoints(spData, fun = function(dat){dat == 1})   ##convert raster to points
-      vertices <- chull(e[,1], e[,2])
-      if(length(vertices) < 3) return(0)
-      vertices <- c(vertices, vertices[1])
-      vertices <- e[vertices,c(1,2)]
-      area = geosphere::areaPolygon(vertices)/1000000
+    if(!all(raster::as.matrix(spData) == floor(raster::as.matrix(spData)), na.rm = TRUE)){ #if probabilistic map
+      upMap <- reclassify(spData, matrix(c(0,0.025,0,0.025,1,1), ncol = 3, byrow = TRUE))
+      consensusMap <- reclassify(spData, matrix(c(0,0.499,0,0.499,1,1), ncol = 3, byrow = TRUE))
+      downMap <- reclassify(spData, matrix(c(0,0.975,0,0.975,1,1), ncol = 3, byrow = TRUE))
+      area <- c(eoo(downMap), eoo(consensusMap), eoo(upMap))
     } else {
-      spData[spData < 1] <- NA
-      spData <- rasterToPoints(spData)
-      vertices <- chull(spData)
-      if(length(vertices) < 3) return(0)
-      vertices <- c(vertices, vertices[1])
-      vertices <- spData[vertices,]
-      area = 0
-      for(i in 1:(nrow(vertices)-1))
-        area = area + (as.numeric(vertices[i,1])*as.numeric(vertices[(i+1),2]) - as.numeric(vertices[i,2])*as.numeric(vertices[(i+1),1]))
-      area = abs(area/2000000)
+      if (raster::xmax(spData) <= 180) {  #if longlat data
+        e <- rasterToPoints(spData, fun = function(dat){dat == 1})   ##convert raster to points
+        vertices <- chull(e[,1], e[,2])
+        if(length(vertices) < 3) return(0)
+        vertices <- c(vertices, vertices[1])
+        vertices <- e[vertices,c(1,2)]
+        area = geosphere::areaPolygon(vertices)/1000000
+      } else {
+        spData[spData < 1] <- NA
+        spData <- rasterToPoints(spData)
+        vertices <- chull(spData)
+        if(length(vertices) < 3) return(0)
+        vertices <- c(vertices, vertices[1])
+        vertices <- spData[vertices,]
+        area = 0
+        for(i in 1:(nrow(vertices)-1))
+          area = area + (as.numeric(vertices[i,1])*as.numeric(vertices[(i+1),2]) - as.numeric(vertices[i,2])*as.numeric(vertices[(i+1),1]))
+        area = abs(area/2000000)
+      }
     }
   } else if (ncol(spData) == 2){
     vertices <- chull(spData)
@@ -1224,37 +1119,44 @@ eoo <- function(spData){
 
 #' Area of Occupancy (AOO).
 #' @description Calculates the Area of Occupancy of a species based on either known records or predicted distribution.
-#' @param spData One of three options: 1) matrix of longitude and latitude (two columns) of each occurrence record; 2) matrix of easting and northing (two columns, e.g. UTM) of each occurrence record in meters;  3) RasterLayer object of predicted distribution (0/1 values).
+#' @param spData One of three options: 1) matrix of longitude and latitude (two columns) of each occurrence record; 2) matrix of easting and northing (two columns, e.g. UTM) of each occurrence record in meters;  3) RasterLayer object of predicted distribution (either 0/1 or probabilistic values).
 #' @details AOO is calculated as the area of all known or predicted cells for the species. The resolution will be 2x2km as required by IUCN.
-#' @return A single value in km2.
+#' @return A single value in km2 or a vector with lower confidence limit, consensus and upper confidence limit (probabilities 0.975, 0.5 and 0.025 respectively).
 #' @examples data(red.range)
 #' aoo(red.range)
 #' @export
 aoo <- function(spData){
-  if (class(spData) == "RasterLayer"){ #if rasterlayer with 0/1
+  if (class(spData) == "RasterLayer"){ #if rasterlayer
     if(raster::maxValue(spData) == 0){  #if no data (empty raster)
       area = 0
-    } else if (raster::xmax(spData) <= 180) {  #if longlat data
-      if(res(spData)[1] > 0.05){ #if resolution is > 1km use area of cells rounded to nearest 4km
-        area = round(cellStats((raster::area(spData) * spData), sum)/4)*4
-      } else {
+    } else if(!all(raster::as.matrix(spData) == floor(raster::as.matrix(spData)), na.rm = TRUE)){ #if probabilistic map
+      upMap <- reclassify(spData, matrix(c(0,0.025,0,0.025,1,1), ncol = 3, byrow = TRUE))
+      consensusMap <- reclassify(spData, matrix(c(0,0.499,0,0.499,1,1), ncol = 3, byrow = TRUE))
+      downMap <- reclassify(spData, matrix(c(0,0.975,0,0.975,1,1), ncol = 3, byrow = TRUE))
+      area <- c(aoo(downMap), aoo(consensusMap), aoo(upMap))
+    } else {
+      if (raster::xmax(spData) <= 180) {  #if longlat data
+        if(res(spData)[1] > 0.05){ #if resolution is > 1km use area of cells rounded to nearest 4km
+          area = round(cellStats((raster::area(spData) * spData), sum)/4)*4
+        } else {
+          spData[spData < 1] <- NA
+          spData <- rasterToPoints(spData)
+          if(nrow(unique(spData)) == 1){
+            area = 4
+          } else {
+            spData <- longlat2utm(spData[,-3])
+            spData = floor(spData/2000)
+            ncells = nrow(unique(spData))
+            area = ncells * 4
+          }
+        }
+      } else { #if square data in meters
         spData[spData < 1] <- NA
         spData <- rasterToPoints(spData)
-        if(nrow(unique(spData)) == 1){
-          area = 4
-        } else {
-          spData <- longlat2utm(spData[,-3])
-          spData = floor(spData/2000)
-          ncells = nrow(unique(spData))
-          area = ncells * 4
-        }
+        spData = floor(spData/2000)
+        ncells = nrow(unique(spData))
+        area = ncells * 4
       }
-    } else { #if square data in meters
-      spData[spData < 1] <- NA
-      spData <- rasterToPoints(spData)
-      spData = floor(spData/2000)
-      ncells = nrow(unique(spData))
-      area = ncells * 4
     }
   } else if (ncol(spData) == 2){
     if (max(spData) <= 180) {  #if longlat data
@@ -1384,12 +1286,14 @@ kml <- function(spData, zone = NULL, filename, mapoption = "aoo", rad = 0.1){
 #' Red List Index.
 #' @description Calculates the Red List Index (RLI) for a group of species.
 #' @param spData Either a vector with species assessment categories for a single point in time or a matrix with two points in time in different columns (species x date). Values can be text (EX, EW, RE, CR, EN, VU, NT, DD, LC) or numeric (0 for LC, 1 for NT, 2 for VU, 3 for EN, 4 for CR, 5 for RE/EW/EX).
+#' @param tree An hclust or phylo object (used when species are weighted by their unique contribution to phylogenetic or functional diversity).
 #' @param boot If TRUE bootstrapping for statistical significance is performed on both values per date and the trend between dates.
 #' @param runs Number of runs for bootstrapping
 #' @details The IUCN Red List Index (RLI) (Butchart et al. 2004, 2007) reflects overall changes in IUCN Red List status over time of a group of taxa.
 #' The RLI uses weight scores based on the Red List status of each of the assessed species. These scores range from 0 (Least Concern) to Extinct/Extinct in the Wild (5).
 #' Summing these scores across all species and relating them to the worst-case scenario, i.e. all species extinct, gives us an indication of how biodiversity is doing.
-#' Importantly, the RLI is based on true improvements or deteriorations in the status of species, i.e. genuine changes. It excludes category changes resulting from, e.g., new knowledge (Butchart et al. 2007).
+#' Each species weight can further be influenced by how much it uniquely contributes to the phylogenetic or functional diversity of the group (Cardoso et al. in prep.).
+#' To incorporate Importantly, the RLI is based on true improvements or deteriorations in the status of species, i.e. genuine changes. It excludes category changes resulting from, e.g., new knowledge (Butchart et al. 2007).
 #' The RLI approach helps to develop a better understanding of which taxa, regions or ecosystems are declining or improving.
 #' Juslen et al. (2016a, b) suggested the use of bootstrapping to search for statistical significance when comparing taxa or for trends in time of the index and this approach is here implemented.
 #' @return Either a vector (if no two dates are given) or a matrix with the RLI values and, if bootstrap is performed, their confidence limits and significance.
@@ -1404,10 +1308,7 @@ kml <- function(spData, zone = NULL, filename, mapoption = "aoo", rad = 0.1){
 #' rli(rliData)
 #' rli(rliData, boot = TRUE)
 #' @export
-rli <- function (spData, boot = FALSE, runs = 1000){
-  ##RLI with phylogenetic or functional data to be implemented soon
-  ##rli <- function (spData, tree = NULL, boot = FALSE, runs = 1000){
-  tree = NULL   ##to add soon
+rli <- function (spData, tree = NULL, boot = FALSE, runs = 1000){
 
   ##if only one point in time is given
   if(is.null(dim(spData)))
@@ -1442,11 +1343,13 @@ rli <- function (spData, boot = FALSE, runs = 1000){
 #' Red List Index for multiple groups.
 #' @description Calculates the Red List Index (RLI) for multiple groups of species.
 #' @param spData A matrix with group names (first column) and species assessment categories for one or two points in time (remaining columns). Values can be text (EX, EW, RE, CR, EN, VU, NT, DD, LC) or numeric (0 for LC, 1 for NT, 2 for VU, 3 for EN, 4 for CR, 5 for RE/EW/EX).
+#' @param tree A list of hclust or phylo objects, each corresponding to a tree per group (used when species are weighted by their unique contribution to phylogenetic or functional diversity).
 #' @param boot If TRUE bootstrapping for statistical significance is performed on both values per date and the trend between dates.
 #' @param runs Number of runs for bootstrapping
 #' @details The IUCN Red List Index (RLI) (Butchart et al. 2004, 2007) reflects overall changes in IUCN Red List status over time of a group of taxa.
 #' The RLI uses weight scores based on the Red List status of each of the assessed species. These scores range from 0 (Least Concern) to 5 (Extinct/Extinct in the Wild).
 #' Summing these scores across all species and relating them to the worst-case scenario, i.e. all species extinct, gives us an indication of how biodiversity is doing.
+#' Each species weight can further be influenced by how much it uniquely contributes to the phylogenetic or functional diversity of the group (Cardoso et al. in prep.).
 #' Importantly, the RLI is based on true improvements or deteriorations in the status of species, i.e. genuine changes. It excludes category changes resulting from, e.g., new knowledge (Butchart et al. 2007).
 #' The RLI approach helps to develop a better understanding of which taxa, regions or ecosystems are declining or improving.
 #' Juslen et al. (2016a, b) suggested the use of bootstrapping to search for statistical significance when comparing taxa or for trends in time of the index and this approach is here implemented.
@@ -1463,10 +1366,7 @@ rli <- function (spData, boot = FALSE, runs = 1000){
 #' rli.multi(rliData)
 #' rli.multi(rliData, boot = TRUE)
 #' @export
-rli.multi <- function (spData, boot = FALSE, runs = 1000){
-  ##RLI with phylogenetic or functional data to be implemented soon
-  ##rli.multi <- function (spData, tree = NULL, boot = FALSE, runs = 1000){
-  tree = NULL ##to add soon xxx
+rli.multi <- function (spData, tree = NULL, boot = FALSE, runs = 1000){
 
   groups <- unique(spData[,1])
   nGroups <- length(groups)
@@ -1480,18 +1380,18 @@ rli.multi <- function (spData, boot = FALSE, runs = 1000){
   }
   row.names(res) <- groups
   for(g in 1:nGroups){
-    if(is.null(tree)){
-      v <- rli(spData[spData[,1] == groups[g],-1], boot = boot, runs = runs)
-      if(ncol(res) < 13){
-        res[g,] <- v
-        colnames(res) <- colnames(v)
-      } else {
-        res[g,1:4] <- v$Values[,1]
-        res[g,5:8] <- v$Values[,2]
-        res[g,9:12] <- v$Values[,3]
-        res[g,13] <- v$P_change
-      }
+    if(is.null(tree))
+      v <- rli(spData[spData[,1] == groups[g],-1], tree = NULL, boot = boot, runs = runs)
+    else
+      v <- rli(spData[spData[,1] == groups[g],-1], tree[[g]], boot = boot, runs = runs)
+    if(ncol(res) < 13){
+      res[g,] <- v
+      colnames(res) <- colnames(v)
     } else {
+      res[g,1:4] <- v$Values[,1]
+      res[g,5:8] <- v$Values[,2]
+      res[g,9:12] <- v$Values[,3]
+      res[g,13] <- v$P_change
     }
   }
   return(res)
@@ -1500,6 +1400,7 @@ rli.multi <- function (spData, boot = FALSE, runs = 1000){
 #' Sampled Red List Index.
 #' @description Calculates accumulation curve of confidence limits in sampled RLI.
 #' @param spData A vector with species assessment categories for a single point in time. Values can be text (EX, EW, RE, CR, EN, VU, NT, DD, LC) or numeric (0 for LC, 1 for NT, 2 for VU, 3 for EN, 4 for CR, 5 for RE/EW/EX).
+#' @param tree An hclust or phylo object (used when species are weighted by their unique contribution to phylogenetic or functional diversity).
 #' @param p p-value of confidence limits (in a two-tailed test).
 #' @param runs Number of runs for smoothing accumulation curves.
 #' @details The IUCN Red List Index (RLI) (Butchart et al. 2004, 2007) reflects overall changes in IUCN Red List status over time of a group of taxa.
@@ -1514,23 +1415,80 @@ rli.multi <- function (spData, boot = FALSE, runs = 1000){
 #' @examples rliData <- c("LC","LC","EN","EN","EX","EX","LC","CR","CR","EX")
 #' rli.sampled(rliData)
 #' @export
-rli.sampled <- function (spData, p = 0.05, runs = 1000){
-  ##RLI with phylogenetic or functional data to be implemented soon
-  ##rli.sampled <- function (spData, tree = NULL, p = 0.05, runs = 1000){
-  tree = NULL ##to add soon xxx
+rli.sampled <- function (spData, tree = NULL, p = 0.05, runs = 1000){
 
   nSpp <- length(spData)
   accum <- rep(NA, nSpp)
   for(n in 1:nSpp){               #test with n species from the entire set
     diff = rep(NA, runs)          #try runs times each species
     for(r in 1:runs){             #do r runs for each n species
-      diff[r] = abs(rli.calc(spData, tree, FALSE, 1) - rli.calc(sample(spData, n), tree, FALSE, 1))  #calculate absolute difference between true and sampled rli for each run
+      rndComm = rep(NA, nSpp)
+      rndSpp = sample(nSpp, n)
+      rndComm[rndSpp] = spData[rndSpp]
+      diff[r] = abs(rli.calc(spData, tree, FALSE, 1) - rli.calc(rndComm, tree, FALSE, 1))  #calculate absolute difference between true and sampled rli for each run
     }
     accum[n] = quantile(diff, (1-p))
   }
   return(accum)   #returns the accumulation curve of confidence limit of sampled RLI
 }
 
+#' Mapping the Red List Index.
+#' @description Creates a map for the red list index according to species distribution and threat status.
+#' @param spData Either a vector with species assessment categories for a single point in time or a matrix with two points in time in different columns (species x date). Values can be text (EX, EW, RE, CR, EN, VU, NT, DD, LC) or numeric (0 for LC, 1 for NT, 2 for VU, 3 for EN, 4 for CR, 5 for RE/EW/EX).
+#' @param layers Species distributions (0/1), a Raster* object as defined by package raster.
+#' @param layers2 Species distributions (0/1) on the second point in time, a Raster* object as defined by package raster. If there are two dates but no layers2, the distributions are assumed to be kept constant in time.
+#' @param tree An hclust or phylo object (used when species are weighted by their unique contribution to phylogenetic or functional diversity).
+#' @details The IUCN Red List Index (RLI) (Butchart et al. 2004, 2007) reflects overall changes in IUCN Red List status over time of a group of taxa.
+#' The RLI uses weight scores based on the Red List status of each of the assessed species. These scores range from 0 (Least Concern) to Extinct/Extinct in the Wild (5).
+#' Summing these scores across all species and relating them to the worst-case scenario, i.e. all species extinct, gives us an indication of how biodiversity is doing.
+#' Each species weight can further be influenced by how much it uniquely contributes to the phylogenetic or functional diversity of the group (Cardoso et al. in prep.).
+#' @return A RasterLayer with point values  (if a single date is given) or change per cell (if two dates are given).
+#' @references Butchart, S.H.M., Stattersfield, A.J., Bennun, L.A., Shutes, S.M., Akcakaya, H.R., Baillie, J.E.M., Stuart, S.N., Hilton-Taylor, C. & Mace, G.M. (2004) Measuring global trends in the status of biodiversity: Red List Indices for birds. PloS Biology, 2: 2294-2304.
+#' @references Butchart, S.H.M., Akcakaya, H.R., Chanson, J., Baillie, J.E.M., Collen, B., Quader, S., Turner, W.R., Amin, R., Stuart, S.N. & Hilton-Taylor, C. (2007) Improvements to the Red List index. PloS One, 2: e140.
+#' @examples sp1 <- raster::raster(matrix(c(1,1,1,0,0,0,0,0,NA), ncol = 3))
+#' sp2 <- raster::raster(matrix(c(1,0,0,1,0,0,1,0,NA), ncol = 3))
+#' sp3 <- raster::raster(matrix(c(1,0,0,0,0,0,0,0,NA), ncol = 3))
+#' sp4 <- raster::raster(matrix(c(0,1,1,1,1,1,1,1,NA), ncol = 3))
+#' layers <- raster::stack(sp1, sp2, sp3, sp4)
+#' spData <- c("CR","EN","VU","LC")
+#' raster::plot(rli.map(spData, layers))
+#' @export
+rli.map <- function (spData, layers, layers2 = NULL, tree = NULL){
+    
+    if(!is.null(dim(spData))){             #if to calculate change call this same function twice
+        if(is.null(layers2)){
+            layers2 <- layers
+        }
+        map1 <- rli.map(spData[,1], layers = layers, tree = tree)
+        map2 <- rli.map(spData[,2], layers = layers2, tree = tree)
+        return(map2 - map1)
+    }
+
+  #convert rasters to array
+  layers = raster::as.array(layers)
+  
+  #get data for each cell (row by row)
+  cells = matrix(NA, (nrow(layers) * ncol(layers)), dim(layers)[3])
+  i = 0
+  for (r in 1:nrow(layers)){
+    for(c in 1:ncol(layers)){
+        i = i+1
+        cells[i,] = layers[r,c,]
+    }
+  }
+
+  #RLI of each cell
+  rliCells = rep(NA, nrow(cells))
+  for (i in 1:nrow(cells)){
+      rliNA <- ifelse(cells[i,] == 1, spData, NA) #only consider species present in each cell
+      rliCells[i] = rli.calc(rliNA, tree = tree)
+  }
+
+  #create RLI map
+  rliMap = raster::raster(matrix(rliCells, nrow = nrow(layers), byrow = T))
+  return(rliMap)
+}
+
 #' Occurrence records for Hogna maderiana (Walckenaer, 1837).
 #'
 #' Occurrence records for Hogna maderiana (Walckenaer, 1837).
